from __future__ import absolute_import


from .modifiers import *  # noqa: F401,F403
from .base import *  # noqa: F401,F403
from .shapes_3d import *  # noqa: F401,F403
from .shapes_2d import *  # noqa: F401,F403
from .boolean import *  # noqa: F401,F403
from .transformations import *  # noqa: F401,F403
from .custom2dshapes import Custom2dShapes  # noqa: F401,F403

__name__ = 'OpenPySCAD'
__version__ = '0.3.2'
__author__ = 'Takuro Wada'
__email__ = 'taxpon@gmail.com'
__url__ = 'https://github.com/taxpon/openpyscad'
__license__ = 'MIT'
