# -*- coding: utf-8 -*-
from __future__ import absolute_import

import openpyscad.base as base

__all__ = ['Union', 'Difference', 'Intersection']


class Union(base.BaseObject):
    pass


class Difference(base.BaseObject):
    pass


class Intersection(base.BaseObject):
    pass
